@@ -135,7 +135,8 @@ extern "C" {
 #define PROFILE_HFP_AG	"/Profile/HFPAG"
 #define PROFILE_HFP_HF	"/Profile/HFPHF"
 
-#define HSP_HS_DEFAULT_CHANNEL  3
+#define HSP_HS_DEFAULT_CHANNEL      3
+#define HFP_SCO_DEFAULT_DATAPATH    0
 
 #define SOURCE_ID_BLUETOOTH	0x1	/* Bluetooth SIG */
 #define SOURCE_ID_USB		0x2	/* USB Implementer's Forum */
@@ -157,6 +158,8 @@ extern "C" {
 
 #define SPA_BT_NO_BATTERY			((uint8_t)255)
 
+#define MAX_CHANNELS	(SPA_AUDIO_MAX_CHANNELS)
+
 enum spa_bt_media_direction {
 	SPA_BT_MEDIA_SOURCE,
 	SPA_BT_MEDIA_SINK,
@@ -376,6 +379,7 @@ struct spa_bt_adapter {
 	unsigned int has_media1_interface:1;
 	unsigned int le_audio_bcast_supported:1;
 	unsigned int tx_timestamping_supported:1;
+	struct spa_list remote_endpoint_list;
 };
 
 enum spa_bt_form_factor {
@@ -571,6 +575,8 @@ struct spa_bt_device {
 
 	const struct media_codec *preferred_codec;
 	uint32_t preferred_profiles;
+
+	bool disable_dummy_call;
 };
 
 struct spa_bt_device *spa_bt_device_find(struct spa_bt_monitor *monitor, const char *path);
@@ -678,7 +684,7 @@ struct spa_bt_transport {
 	struct spa_list bap_transport_linked;
 
 	uint32_t n_channels;
-	uint32_t channels[SPA_AUDIO_MAX_CHANNELS];
+	uint32_t channels[MAX_CHANNELS];
 
 	struct spa_bt_transport_volume volumes[SPA_BT_VOLUME_ID_TERM];
 
