@@ -739,7 +739,7 @@ static int impl_clear(struct spa_handle *handle)
 	this = (struct impl *) handle;
 
 	if (this->data_loop)
-		spa_loop_invoke(this->data_loop, do_remove_timer, 0, NULL, 0, true, this);
+		spa_loop_locked(this->data_loop, do_remove_timer, 0, NULL, 0, this);
 	spa_system_close(this->data_system, this->timer_source.fd);
 
 	return 0;
