@@ -769,7 +769,7 @@ static GstBuffer *dequeue_buffer(GstPipeWireSrc *pwsrc)
     GST_BUFFER_DTS (buf) = b->time - pwsrc->delay;
   }
 
-  if (pwsrc->is_video) {
+  if (pwsrc->media_type == SPA_MEDIA_TYPE_video) {
     if (pwsrc->video_info.fps_n) {
       GST_BUFFER_DURATION (buf) = gst_util_uint64_scale (GST_SECOND,
           pwsrc->video_info.fps_d, pwsrc->video_info.fps_n);
@@ -781,7 +781,7 @@ static GstBuffer *dequeue_buffer(GstPipeWireSrc *pwsrc)
 
   crop = data->crop;
   if (crop) {
-    GstVideoCropMeta *meta = gst_buffer_get_video_crop_meta(buf);
+    GstVideoCropMeta *meta = gst_buffer_add_video_crop_meta(buf);
     if (meta) {
       meta->x = crop->region.position.x;
       meta->y = crop->region.position.y;
@@ -912,7 +912,7 @@ on_state_changed (void *data,
        * must handle the clock lost message in it's bus handler by pausing
        * the pipeline and then setting it back to playing.
        */
-      if (current_state == GST_STATE_PLAYING && !pwsrc->is_video)
+      if (current_state == GST_STATE_PLAYING && pwsrc->media_type == SPA_MEDIA_TYPE_audio)
         gst_element_post_message (GST_ELEMENT_CAST (pwsrc),
             gst_message_new_clock_lost (GST_OBJECT_CAST (pwsrc),
                 GST_CLOCK_CAST (pwsrc->stream->clock)));
@@ -1056,6 +1056,44 @@ wait_started (GstPipeWireSrc *this)
   return state;
 }
 
+
+static enum pw_stream_state
+wait_negotiated (GstPipeWireSrc *this)
+{
+  enum pw_stream_state state;
+  const char *error = NULL;
+  struct timespec abstime;
+
+  pw_thread_loop_get_time (this->stream->core->loop, &abstime,
+                  GST_PIPEWIRE_DEFAULT_TIMEOUT * SPA_NSEC_PER_SEC);
+
+  while (TRUE) {
+    state = pw_stream_get_state (this->stream->pwstream, &error);
+
+    GST_DEBUG_OBJECT (this, "waiting for NEGOTIATED, now %s", pw_stream_state_as_string (state));
+    if (state == PW_STREAM_STATE_ERROR)
+      break;
+    if (this->flushing) {
+      state = PW_STREAM_STATE_ERROR;
+      break;
+    }
+
+    if (this->negotiated)
+      break;
+
+    if (this->autoconnect) {
+      if (pw_thread_loop_timed_wait_full (this->stream->core->loop, &abstime) < 0) {
+        state = PW_STREAM_STATE_ERROR;
+        break;
+      }
+    } else {
+      pw_thread_loop_wait (this->stream->core->loop);
+    }
+  }
+  GST_DEBUG_OBJECT (this, state != PW_STREAM_STATE_ERROR ? "got negotiated signal" : "error during negotiation");
+  return state;
+}
+
 static gboolean
 gst_pipewire_src_negotiate (GstBaseSrc * basesrc)
 {
@@ -1067,7 +1105,6 @@ gst_pipewire_src_negotiate (GstBaseSrc * basesrc)
   g_autoptr (GPtrArray) possible = NULL;
   gboolean result = FALSE;
   const char *error = NULL;
-  struct timespec abstime;
   uint32_t target_id;
 
   /* first see what is possible on our source pad */
@@ -1177,26 +1214,8 @@ gst_pipewire_src_negotiate (GstBaseSrc * basesrc)
                      (const struct spa_pod **)possible->pdata,
                      possible->len);
 
-  pw_thread_loop_get_time (pwsrc->stream->core->loop, &abstime,
-                  GST_PIPEWIRE_DEFAULT_TIMEOUT * SPA_NSEC_PER_SEC);
-
-  while (TRUE) {
-    enum pw_stream_state state = pw_stream_get_state (pwsrc->stream->pwstream, &error);
-
-    GST_DEBUG_OBJECT (basesrc, "waiting for NEGOTIATED, now %s", pw_stream_state_as_string (state));
-    if (state == PW_STREAM_STATE_ERROR || pwsrc->flushing)
-      goto connect_error;
-
-    if (pwsrc->negotiated)
-      break;
-
-    if (pwsrc->autoconnect) {
-      if (pw_thread_loop_timed_wait_full (pwsrc->stream->core->loop, &abstime) < 0)
-        goto connect_error;
-    } else {
-      pw_thread_loop_wait (pwsrc->stream->core->loop);
-    }
-  }
+  if (wait_negotiated(pwsrc) == PW_STREAM_STATE_ERROR)
+    goto connect_error;
 
   negotiated_caps = g_steal_pointer (&pwsrc->caps);
   pw_thread_loop_unlock (pwsrc->stream->core->loop);
@@ -1265,7 +1284,7 @@ handle_format_change (GstPipeWireSrc *pwsrc,
   if (param == NULL) {
     GST_DEBUG_OBJECT (pwsrc, "clear format");
     pwsrc->negotiated = FALSE;
-    pwsrc->is_video = FALSE;
+    pwsrc->media_type = SPA_MEDIA_TYPE_unknown;
     return;
   }
 
@@ -1303,7 +1322,7 @@ handle_format_change (GstPipeWireSrc *pwsrc,
     structure = gst_caps_get_structure (pwsrc->caps, 0);
     if (g_str_has_prefix (gst_structure_get_name (structure), "video/") ||
         g_str_has_prefix (gst_structure_get_name (structure), "image/")) {
-      pwsrc->is_video = TRUE;
+      pwsrc->media_type = SPA_MEDIA_TYPE_video;
 
 #ifdef HAVE_GSTREAMER_DMA_DRM
       if (gst_video_is_dma_drm_caps (pwsrc->caps)) {
@@ -1342,10 +1361,12 @@ handle_format_change (GstPipeWireSrc *pwsrc,
        * application/user */
       if (pwsrc->use_bufferpool != USE_BUFFERPOOL_YES)
         pwsrc->use_bufferpool = USE_BUFFERPOOL_NO;
+
+      pwsrc->media_type = SPA_MEDIA_TYPE_audio;
     }
   } else {
     pwsrc->negotiated = FALSE;
-    pwsrc->is_video = FALSE;
+    pwsrc->media_type = SPA_MEDIA_TYPE_unknown;
     pwsrc->is_rawvideo = FALSE;
   }
 
@@ -1722,12 +1743,21 @@ gst_pipewire_src_change_state (GstElement * element, GstStateChange transition)
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       /* uncork and start recording */
+      GST_DEBUG_OBJECT (this, "activating stream");
+
       pw_thread_loop_lock (this->stream->core->loop);
       pw_stream_set_active (this->stream->pwstream, true);
+      /* if state have been paused for longer time, the underlying node might
+       * be moved from idle to suspended, which would mean format cleared via
+       * handle_format_change. Wait for new format to avoid basesrc calling
+       * create() and get not-negotiated error as response. */
+      if (wait_negotiated(this) == PW_STREAM_STATE_ERROR)
+        goto open_failed;
       pw_thread_loop_unlock (this->stream->core->loop);
       break;
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       /* stop recording ASAP by corking */
+      GST_DEBUG_OBJECT (this, "in-activating stream");
       pw_thread_loop_lock (this->stream->core->loop);
       pw_stream_set_active (this->stream->pwstream, false);
       pw_thread_loop_unlock (this->stream->core->loop);
