@@ -11,6 +11,7 @@
 #include <spa/buffer/alloc.h>
 #include <spa/param/props.h>
 #include <spa/param/format-utils.h>
+#include <spa/param/peer-utils.h>
 #include <spa/node/io.h>
 #include <spa/node/utils.h>
 #include <spa/utils/cleanup.h>
@@ -103,14 +104,16 @@ struct stream {
 	uint64_t port_change_mask_all;
 	struct spa_port_info port_info;
 	struct pw_properties *port_props;
-#define PORT_EnumFormat	0
-#define PORT_Meta	1
-#define PORT_IO		2
-#define PORT_Format	3
-#define PORT_Buffers	4
-#define PORT_Latency	5
-#define PORT_Tag	6
-#define N_PORT_PARAMS	7
+#define PORT_EnumFormat		0
+#define PORT_Meta		1
+#define PORT_IO			2
+#define PORT_Format		3
+#define PORT_Buffers		4
+#define PORT_Latency		5
+#define PORT_Tag		6
+#define PORT_Capability		7
+#define PORT_PeerCapability	8
+#define N_PORT_PARAMS		9
 	struct spa_param_info port_params[N_PORT_PARAMS];
 
 	struct spa_list param_list;
@@ -144,6 +147,7 @@ struct stream {
 
 	struct spa_callbacks rt_callbacks;
 
+	unsigned int have_peer_capability:1;
 	unsigned int disconnecting:1;
 	unsigned int disconnect_core:1;
 	unsigned int draining:1;
@@ -196,6 +200,10 @@ static int get_port_param_index(uint32_t id)
 		return PORT_Latency;
 	case SPA_PARAM_Tag:
 		return PORT_Tag;
+	case SPA_PARAM_Capability:
+		return PORT_Capability;
+	case SPA_PARAM_PeerCapability:
+		return PORT_PeerCapability;
 	default:
 		return -1;
 	}
@@ -707,8 +715,6 @@ static int impl_send_command(void *object, const struct spa_command *command)
 	case SPA_NODE_COMMAND_Suspend:
 	case SPA_NODE_COMMAND_Flush:
 	case SPA_NODE_COMMAND_Pause:
-		pw_loop_invoke(impl->main_loop,
-			NULL, 0, NULL, 0, false, impl);
 		if (stream->state == PW_STREAM_STATE_STREAMING && id != SPA_NODE_COMMAND_Flush) {
 			pw_log_debug("%p: pause", stream);
 			stream_set_state(stream, PW_STREAM_STATE_PAUSED, 0, NULL);
@@ -909,6 +915,21 @@ static int parse_latency(struct pw_stream *stream, const struct spa_pod *param,
 	return 0;
 }
 
+static void emit_dummy_peer_capability(struct stream *impl, bool empty)
+{
+	struct spa_pod *param = NULL;
+	uint8_t buffer[1024];
+	if (!empty) {
+		struct spa_pod_frame f;
+		struct spa_pod_builder b;
+		spa_pod_builder_init(&b, buffer, sizeof(buffer));
+		spa_peer_param_build_start(&b, &f, SPA_PARAM_PeerCapability);
+		spa_peer_param_build_add_param(&b, SPA_ID_INVALID, NULL);
+		param = spa_peer_param_build_end(&b, &f);
+	}
+	emit_param_changed(impl, SPA_PARAM_PeerCapability, param);
+}
+
 static int impl_port_set_param(void *object,
 			       enum spa_direction direction, uint32_t port_id,
 			       uint32_t id, uint32_t flags,
@@ -916,7 +937,7 @@ static int impl_port_set_param(void *object,
 {
 	struct stream *impl = object;
 	struct pw_stream *stream = &impl->this;
-	uint32_t user, fl = 0;
+	uint32_t user = 0, fl = 0;
 	int res;
 	const struct spa_pod *params[1];
 	uint32_t n_params = 0;
@@ -936,7 +957,12 @@ static int impl_port_set_param(void *object,
 	n_params = param ? 1 : 0;
 
 	switch (id) {
+	case SPA_PARAM_PeerCapability:
+		impl->have_peer_capability = true;
+		break;
 	case SPA_PARAM_Latency:
+		if (!impl->have_peer_capability)
+			emit_dummy_peer_capability(impl, param == NULL);
 		parse_latency(stream, param, &fl);
 		break;
 	}
@@ -1270,6 +1296,9 @@ static int node_event_param(void *object, int seq,
 			return 0;
 
 		c = calloc(1, sizeof(*c) + SPA_POD_SIZE(param));
+		if (c == NULL)
+			return -errno;
+
 		c->info = SPA_PTROFF(c, sizeof(*c), struct spa_pod);
 		memcpy(c->info, param, SPA_POD_SIZE(param));
 		c->control.n_values = 0;
@@ -1286,7 +1315,7 @@ static int node_event_param(void *object, int seq,
 		}
 
 		pod = spa_pod_get_values(type, &n_vals, &choice);
-		if (n_vals == 0) {
+		if (n_vals < 1) {
 			free(c);
 			return -EINVAL;
 		}
@@ -1318,19 +1347,12 @@ static int node_event_param(void *object, int seq,
 
 		switch (choice) {
 		case SPA_CHOICE_None:
-			if (n_vals < 1) {
-				free(c);
-				return -EINVAL;
-			}
 			c->control.n_values = 1;
 			c->control.max_values = 1;
 			c->control.values[0] = c->control.def = c->control.min = c->control.max = vals[0];
 			break;
 		case SPA_CHOICE_Range:
-			if (n_vals < 3) {
-				free(c);
-				return -EINVAL;
-			}
+		case SPA_CHOICE_Step:
 			c->control.n_values = 1;
 			c->control.max_values = 1;
 			c->control.values[0] = vals[0];
@@ -1724,11 +1746,35 @@ static int stream_disconnect(struct stream *impl)
 	return 0;
 }
 
+static void stream_free(struct pw_stream *stream)
+{
+	struct stream *impl = SPA_CONTAINER_OF(stream, struct stream, this);
+	struct control *c;
+
+	pw_log_debug("%p: free", stream);
+	clear_params(impl, SPA_ID_INVALID, 0);
+
+	free(stream->error);
+
+	pw_properties_free(stream->properties);
+
+	free(stream->name);
+
+	spa_list_consume(c, &stream->controls, link) {
+		spa_list_remove(&c->link);
+		free(c);
+	}
+	if (impl->data.context)
+		pw_context_destroy(impl->data.context);
+
+	pw_properties_free(impl->port_props);
+	free(impl);
+}
+
 SPA_EXPORT
 void pw_stream_destroy(struct pw_stream *stream)
 {
 	struct stream *impl = SPA_CONTAINER_OF(stream, struct stream, this);
-	struct control *c;
 
 	ensure_loop(impl->main_loop, return);
 
@@ -1744,29 +1790,13 @@ void pw_stream_destroy(struct pw_stream *stream)
 		spa_list_remove(&stream->link);
 		stream->core = NULL;
 	}
-
-	clear_params(impl, SPA_ID_INVALID, 0);
-
-	pw_log_debug("%p: free", stream);
-	free(stream->error);
-
-	pw_properties_free(stream->properties);
-
-	free(stream->name);
-
-	spa_list_consume(c, &stream->controls, link) {
-		spa_list_remove(&c->link);
-		free(c);
-	}
-
 	spa_hook_list_clean(&impl->hooks);
 	spa_hook_list_clean(&stream->listener_list);
 
-	if (impl->data.context)
-		pw_context_destroy(impl->data.context);
+	/* Make sure there are no queued invokes from us anymore */
+	pw_loop_invoke(impl->main_loop, NULL, 0, NULL, 0, false, impl);
 
-	pw_properties_free(impl->port_props);
-	free(impl);
+	stream_free(stream);
 }
 
 static int
@@ -2019,6 +2049,8 @@ pw_stream_connect(struct pw_stream *stream,
 	impl->port_params[PORT_Buffers] = SPA_PARAM_INFO(SPA_PARAM_Buffers, 0);
 	impl->port_params[PORT_Latency] = SPA_PARAM_INFO(SPA_PARAM_Latency, SPA_PARAM_INFO_WRITE);
 	impl->port_params[PORT_Tag] = SPA_PARAM_INFO(SPA_PARAM_Tag, SPA_PARAM_INFO_WRITE);
+	impl->port_params[PORT_Capability] = SPA_PARAM_INFO(SPA_PARAM_Capability, 0);
+	impl->port_params[PORT_PeerCapability] = SPA_PARAM_INFO(SPA_PARAM_PeerCapability, SPA_PARAM_INFO_WRITE);
 	impl->port_info.props = &impl->port_props->dict;
 	impl->port_info.params = impl->port_params;
 	impl->port_info.n_params = N_PORT_PARAMS;
@@ -2449,8 +2481,9 @@ int pw_stream_get_time_n(struct pw_stream *stream, struct pw_time *time, size_t
 
 	time->delay += (int64_t)(((latency->min_quantum + latency->max_quantum) / 2.0f) * quantum);
 	time->delay += (latency->min_rate + latency->max_rate) / 2;
-	time->delay += ((latency->min_ns + latency->max_ns) / 2) *
-		(int64_t)time->rate.denom / (int64_t)SPA_NSEC_PER_SEC;
+	if (time->rate.num != 0)
+		time->delay += ((latency->min_ns + latency->max_ns) / 2) *
+			(int64_t)time->rate.denom / ((int64_t)SPA_NSEC_PER_SEC * time->rate.num);
 
 	avail_buffers = spa_ringbuffer_get_read_index(&impl->dequeued.ring, &index);
 	avail_buffers = SPA_CLAMP(avail_buffers, 0, (int32_t)impl->n_buffers);
@@ -2471,7 +2504,8 @@ int pw_stream_get_time_n(struct pw_stream *stream, struct pw_time *time, size_t
 			impl->dequeued.outcount, impl->dequeued.incount,
 			impl->queued.outcount, impl->queued.incount,
 			avail_buffers, impl->n_buffers);
-	return 0;
+
+	return stream->state == PW_STREAM_STATE_STREAMING ? 0 : -EIO;
 }
 
 SPA_EXPORT
